/****************************************************************************
** Meta object code from reading C++ file 'multipageloader.hh'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "multipageloader.hh"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'multipageloader.hh' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_wkhtmltopdf__MultiPageLoader_t {
    QByteArrayData data[13];
    char stringdata0[126];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_wkhtmltopdf__MultiPageLoader_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_wkhtmltopdf__MultiPageLoader_t qt_meta_stringdata_wkhtmltopdf__MultiPageLoader = {
    {
QT_MOC_LITERAL(0, 0, 28), // "wkhtmltopdf::MultiPageLoader"
QT_MOC_LITERAL(1, 29, 12), // "loadFinished"
QT_MOC_LITERAL(2, 42, 0), // ""
QT_MOC_LITERAL(3, 43, 2), // "ok"
QT_MOC_LITERAL(4, 46, 12), // "loadProgress"
QT_MOC_LITERAL(5, 59, 8), // "progress"
QT_MOC_LITERAL(6, 68, 11), // "loadStarted"
QT_MOC_LITERAL(7, 80, 7), // "warning"
QT_MOC_LITERAL(8, 88, 4), // "text"
QT_MOC_LITERAL(9, 93, 5), // "error"
QT_MOC_LITERAL(10, 99, 4), // "load"
QT_MOC_LITERAL(11, 104, 14), // "clearResources"
QT_MOC_LITERAL(12, 119, 6) // "cancel"

    },
    "wkhtmltopdf::MultiPageLoader\0loadFinished\0"
    "\0ok\0loadProgress\0progress\0loadStarted\0"
    "warning\0text\0error\0load\0clearResources\0"
    "cancel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_wkhtmltopdf__MultiPageLoader[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   54,    2, 0x06 /* Public */,
       4,    1,   57,    2, 0x06 /* Public */,
       6,    0,   60,    2, 0x06 /* Public */,
       7,    1,   61,    2, 0x06 /* Public */,
       9,    1,   64,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,   67,    2, 0x0a /* Public */,
      11,    0,   68,    2, 0x0a /* Public */,
      12,    0,   69,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString,    8,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void wkhtmltopdf::MultiPageLoader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MultiPageLoader *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->loadFinished((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->loadProgress((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->loadStarted(); break;
        case 3: _t->warning((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 4: _t->error((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 5: _t->load(); break;
        case 6: _t->clearResources(); break;
        case 7: _t->cancel(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MultiPageLoader::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MultiPageLoader::loadFinished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MultiPageLoader::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MultiPageLoader::loadProgress)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (MultiPageLoader::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MultiPageLoader::loadStarted)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (MultiPageLoader::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MultiPageLoader::warning)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (MultiPageLoader::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MultiPageLoader::error)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject wkhtmltopdf::MultiPageLoader::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_wkhtmltopdf__MultiPageLoader.data,
    qt_meta_data_wkhtmltopdf__MultiPageLoader,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *wkhtmltopdf::MultiPageLoader::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *wkhtmltopdf::MultiPageLoader::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_wkhtmltopdf__MultiPageLoader.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int wkhtmltopdf::MultiPageLoader::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void wkhtmltopdf::MultiPageLoader::loadFinished(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void wkhtmltopdf::MultiPageLoader::loadProgress(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void wkhtmltopdf::MultiPageLoader::loadStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void wkhtmltopdf::MultiPageLoader::warning(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void wkhtmltopdf::MultiPageLoader::error(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
