/****************************************************************************
** Meta object code from reading C++ file 'converter.hh'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../lib/converter.hh"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'converter.hh' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_wkhtmltopdf__Converter_t {
    QByteArrayData data[18];
    char stringdata0[228];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_wkhtmltopdf__Converter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_wkhtmltopdf__Converter_t qt_meta_stringdata_wkhtmltopdf__Converter = {
    {
QT_MOC_LITERAL(0, 0, 22), // "wkhtmltopdf::Converter"
QT_MOC_LITERAL(1, 23, 7), // "warning"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 7), // "message"
QT_MOC_LITERAL(4, 40, 5), // "error"
QT_MOC_LITERAL(5, 46, 12), // "phaseChanged"
QT_MOC_LITERAL(6, 59, 15), // "progressChanged"
QT_MOC_LITERAL(7, 75, 8), // "progress"
QT_MOC_LITERAL(8, 84, 8), // "finished"
QT_MOC_LITERAL(9, 93, 2), // "ok"
QT_MOC_LITERAL(10, 96, 18), // "checkboxSvgChanged"
QT_MOC_LITERAL(11, 115, 4), // "path"
QT_MOC_LITERAL(12, 120, 25), // "checkboxCheckedSvgChanged"
QT_MOC_LITERAL(13, 146, 21), // "radiobuttonSvgChanged"
QT_MOC_LITERAL(14, 168, 28), // "radiobuttonCheckedSvgChanged"
QT_MOC_LITERAL(15, 197, 15), // "beginConversion"
QT_MOC_LITERAL(16, 213, 7), // "convert"
QT_MOC_LITERAL(17, 221, 6) // "cancel"

    },
    "wkhtmltopdf::Converter\0warning\0\0message\0"
    "error\0phaseChanged\0progressChanged\0"
    "progress\0finished\0ok\0checkboxSvgChanged\0"
    "path\0checkboxCheckedSvgChanged\0"
    "radiobuttonSvgChanged\0"
    "radiobuttonCheckedSvgChanged\0"
    "beginConversion\0convert\0cancel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_wkhtmltopdf__Converter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       4,    1,   77,    2, 0x06 /* Public */,
       5,    0,   80,    2, 0x06 /* Public */,
       6,    1,   81,    2, 0x06 /* Public */,
       8,    1,   84,    2, 0x06 /* Public */,
      10,    1,   87,    2, 0x06 /* Public */,
      12,    1,   90,    2, 0x06 /* Public */,
      13,    1,   93,    2, 0x06 /* Public */,
      14,    1,   96,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      15,    0,   99,    2, 0x0a /* Public */,
      16,    0,  100,    2, 0x0a /* Public */,
      17,    0,  101,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Bool,    9,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void, QMetaType::QString,   11,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,

       0        // eod
};

void wkhtmltopdf::Converter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Converter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->warning((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->error((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->phaseChanged(); break;
        case 3: _t->progressChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->finished((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->checkboxSvgChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->checkboxCheckedSvgChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->radiobuttonSvgChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->radiobuttonCheckedSvgChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->beginConversion(); break;
        case 10: { bool _r = _t->convert();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->cancel(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Converter::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Converter::warning)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Converter::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Converter::error)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Converter::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Converter::phaseChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (Converter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Converter::progressChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (Converter::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Converter::finished)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (Converter::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Converter::checkboxSvgChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (Converter::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Converter::checkboxCheckedSvgChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (Converter::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Converter::radiobuttonSvgChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (Converter::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Converter::radiobuttonCheckedSvgChanged)) {
                *result = 8;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject wkhtmltopdf::Converter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_wkhtmltopdf__Converter.data,
    qt_meta_data_wkhtmltopdf__Converter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *wkhtmltopdf::Converter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *wkhtmltopdf::Converter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_wkhtmltopdf__Converter.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int wkhtmltopdf::Converter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void wkhtmltopdf::Converter::warning(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void wkhtmltopdf::Converter::error(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void wkhtmltopdf::Converter::phaseChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void wkhtmltopdf::Converter::progressChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void wkhtmltopdf::Converter::finished(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void wkhtmltopdf::Converter::checkboxSvgChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void wkhtmltopdf::Converter::checkboxCheckedSvgChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void wkhtmltopdf::Converter::radiobuttonSvgChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void wkhtmltopdf::Converter::radiobuttonCheckedSvgChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
