/****************************************************************************
** Meta object code from reading C++ file 'multipageloader_p.hh'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../lib/multipageloader_p.hh"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'multipageloader_p.hh' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_wkhtmltopdf__MyNetworkProxyFactory_t {
    QByteArrayData data[1];
    char stringdata0[35];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_wkhtmltopdf__MyNetworkProxyFactory_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_wkhtmltopdf__MyNetworkProxyFactory_t qt_meta_stringdata_wkhtmltopdf__MyNetworkProxyFactory = {
    {
QT_MOC_LITERAL(0, 0, 34) // "wkhtmltopdf::MyNetworkProxyFa..."

    },
    "wkhtmltopdf::MyNetworkProxyFactory"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_wkhtmltopdf__MyNetworkProxyFactory[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void wkhtmltopdf::MyNetworkProxyFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject wkhtmltopdf::MyNetworkProxyFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_wkhtmltopdf__MyNetworkProxyFactory.data,
    qt_meta_data_wkhtmltopdf__MyNetworkProxyFactory,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *wkhtmltopdf::MyNetworkProxyFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *wkhtmltopdf::MyNetworkProxyFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_wkhtmltopdf__MyNetworkProxyFactory.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QNetworkProxyFactory"))
        return static_cast< QNetworkProxyFactory*>(this);
    return QObject::qt_metacast(_clname);
}

int wkhtmltopdf::MyNetworkProxyFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_wkhtmltopdf__MyNetworkAccessManager_t {
    QByteArrayData data[5];
    char stringdata0[56];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_wkhtmltopdf__MyNetworkAccessManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_wkhtmltopdf__MyNetworkAccessManager_t qt_meta_stringdata_wkhtmltopdf__MyNetworkAccessManager = {
    {
QT_MOC_LITERAL(0, 0, 35), // "wkhtmltopdf::MyNetworkAccessM..."
QT_MOC_LITERAL(1, 36, 7), // "warning"
QT_MOC_LITERAL(2, 44, 0), // ""
QT_MOC_LITERAL(3, 45, 4), // "text"
QT_MOC_LITERAL(4, 50, 5) // "error"

    },
    "wkhtmltopdf::MyNetworkAccessManager\0"
    "warning\0\0text\0error"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_wkhtmltopdf__MyNetworkAccessManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,
       4,    1,   27,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,

       0        // eod
};

void wkhtmltopdf::MyNetworkAccessManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MyNetworkAccessManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->warning((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->error((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MyNetworkAccessManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MyNetworkAccessManager::warning)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (MyNetworkAccessManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MyNetworkAccessManager::error)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject wkhtmltopdf::MyNetworkAccessManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QNetworkAccessManager::staticMetaObject>(),
    qt_meta_stringdata_wkhtmltopdf__MyNetworkAccessManager.data,
    qt_meta_data_wkhtmltopdf__MyNetworkAccessManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *wkhtmltopdf::MyNetworkAccessManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *wkhtmltopdf::MyNetworkAccessManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_wkhtmltopdf__MyNetworkAccessManager.stringdata0))
        return static_cast<void*>(this);
    return QNetworkAccessManager::qt_metacast(_clname);
}

int wkhtmltopdf::MyNetworkAccessManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QNetworkAccessManager::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void wkhtmltopdf::MyNetworkAccessManager::warning(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void wkhtmltopdf::MyNetworkAccessManager::error(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_wkhtmltopdf__MyQWebPage_t {
    QByteArrayData data[3];
    char stringdata0[51];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_wkhtmltopdf__MyQWebPage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_wkhtmltopdf__MyQWebPage_t qt_meta_stringdata_wkhtmltopdf__MyQWebPage = {
    {
QT_MOC_LITERAL(0, 0, 23), // "wkhtmltopdf::MyQWebPage"
QT_MOC_LITERAL(1, 24, 25), // "shouldInterruptJavaScript"
QT_MOC_LITERAL(2, 50, 0) // ""

    },
    "wkhtmltopdf::MyQWebPage\0"
    "shouldInterruptJavaScript\0"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_wkhtmltopdf__MyQWebPage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       1,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   19,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Bool,

       0        // eod
};

void wkhtmltopdf::MyQWebPage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MyQWebPage *>(_o);
        (void)_t;
        switch (_id) {
        case 0: { bool _r = _t->shouldInterruptJavaScript();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject wkhtmltopdf::MyQWebPage::staticMetaObject = { {
    QMetaObject::SuperData::link<QWebPage::staticMetaObject>(),
    qt_meta_stringdata_wkhtmltopdf__MyQWebPage.data,
    qt_meta_data_wkhtmltopdf__MyQWebPage,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *wkhtmltopdf::MyQWebPage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *wkhtmltopdf::MyQWebPage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_wkhtmltopdf__MyQWebPage.stringdata0))
        return static_cast<void*>(this);
    return QWebPage::qt_metacast(_clname);
}

int wkhtmltopdf::MyQWebPage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWebPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 1;
    }
    return _id;
}
struct qt_meta_stringdata_wkhtmltopdf__ResourceObject_t {
    QByteArrayData data[24];
    char stringdata0[278];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_wkhtmltopdf__ResourceObject_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_wkhtmltopdf__ResourceObject_t qt_meta_stringdata_wkhtmltopdf__ResourceObject = {
    {
QT_MOC_LITERAL(0, 0, 27), // "wkhtmltopdf::ResourceObject"
QT_MOC_LITERAL(1, 28, 4), // "load"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 11), // "loadStarted"
QT_MOC_LITERAL(4, 46, 12), // "loadProgress"
QT_MOC_LITERAL(5, 59, 8), // "progress"
QT_MOC_LITERAL(6, 68, 12), // "loadFinished"
QT_MOC_LITERAL(7, 81, 2), // "ok"
QT_MOC_LITERAL(8, 84, 16), // "waitWindowStatus"
QT_MOC_LITERAL(9, 101, 14), // "printRequested"
QT_MOC_LITERAL(10, 116, 10), // "QWebFrame*"
QT_MOC_LITERAL(11, 127, 5), // "frame"
QT_MOC_LITERAL(12, 133, 8), // "loadDone"
QT_MOC_LITERAL(13, 142, 28), // "handleAuthenticationRequired"
QT_MOC_LITERAL(14, 171, 14), // "QNetworkReply*"
QT_MOC_LITERAL(15, 186, 5), // "reply"
QT_MOC_LITERAL(16, 192, 15), // "QAuthenticator*"
QT_MOC_LITERAL(17, 208, 13), // "authenticator"
QT_MOC_LITERAL(18, 222, 7), // "warning"
QT_MOC_LITERAL(19, 230, 3), // "str"
QT_MOC_LITERAL(20, 234, 5), // "error"
QT_MOC_LITERAL(21, 240, 9), // "sslErrors"
QT_MOC_LITERAL(22, 250, 16), // "QList<QSslError>"
QT_MOC_LITERAL(23, 267, 10) // "amfinished"

    },
    "wkhtmltopdf::ResourceObject\0load\0\0"
    "loadStarted\0loadProgress\0progress\0"
    "loadFinished\0ok\0waitWindowStatus\0"
    "printRequested\0QWebFrame*\0frame\0"
    "loadDone\0handleAuthenticationRequired\0"
    "QNetworkReply*\0reply\0QAuthenticator*\0"
    "authenticator\0warning\0str\0error\0"
    "sslErrors\0QList<QSslError>\0amfinished"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_wkhtmltopdf__ResourceObject[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x0a /* Public */,
       3,    0,   75,    2, 0x0a /* Public */,
       4,    1,   76,    2, 0x0a /* Public */,
       6,    1,   79,    2, 0x0a /* Public */,
       8,    0,   82,    2, 0x0a /* Public */,
       9,    1,   83,    2, 0x0a /* Public */,
      12,    0,   86,    2, 0x0a /* Public */,
      13,    2,   87,    2, 0x0a /* Public */,
      18,    1,   92,    2, 0x0a /* Public */,
      20,    1,   95,    2, 0x0a /* Public */,
      21,    2,   98,    2, 0x0a /* Public */,
      23,    1,  103,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14, 0x80000000 | 16,   15,   17,
    QMetaType::Void, QMetaType::QString,   19,
    QMetaType::Void, QMetaType::QString,   19,
    QMetaType::Void, 0x80000000 | 14, 0x80000000 | 22,   15,    2,
    QMetaType::Void, 0x80000000 | 14,   15,

       0        // eod
};

void wkhtmltopdf::ResourceObject::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ResourceObject *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->load(); break;
        case 1: _t->loadStarted(); break;
        case 2: _t->loadProgress((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->loadFinished((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->waitWindowStatus(); break;
        case 5: _t->printRequested((*reinterpret_cast< QWebFrame*(*)>(_a[1]))); break;
        case 6: _t->loadDone(); break;
        case 7: _t->handleAuthenticationRequired((*reinterpret_cast< QNetworkReply*(*)>(_a[1])),(*reinterpret_cast< QAuthenticator*(*)>(_a[2]))); break;
        case 8: _t->warning((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->error((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->sslErrors((*reinterpret_cast< QNetworkReply*(*)>(_a[1])),(*reinterpret_cast< const QList<QSslError>(*)>(_a[2]))); break;
        case 11: _t->amfinished((*reinterpret_cast< QNetworkReply*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWebFrame* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QNetworkReply* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QSslError> >(); break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QNetworkReply* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QNetworkReply* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject wkhtmltopdf::ResourceObject::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_wkhtmltopdf__ResourceObject.data,
    qt_meta_data_wkhtmltopdf__ResourceObject,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *wkhtmltopdf::ResourceObject::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *wkhtmltopdf::ResourceObject::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_wkhtmltopdf__ResourceObject.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int wkhtmltopdf::ResourceObject::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}
struct qt_meta_stringdata_wkhtmltopdf__MultiPageLoaderPrivate_t {
    QByteArrayData data[4];
    char stringdata0[51];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_wkhtmltopdf__MultiPageLoaderPrivate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_wkhtmltopdf__MultiPageLoaderPrivate_t qt_meta_stringdata_wkhtmltopdf__MultiPageLoaderPrivate = {
    {
QT_MOC_LITERAL(0, 0, 35), // "wkhtmltopdf::MultiPageLoaderP..."
QT_MOC_LITERAL(1, 36, 4), // "fail"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 8) // "loadDone"

    },
    "wkhtmltopdf::MultiPageLoaderPrivate\0"
    "fail\0\0loadDone"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_wkhtmltopdf__MultiPageLoaderPrivate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   24,    2, 0x0a /* Public */,
       3,    0,   25,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void wkhtmltopdf::MultiPageLoaderPrivate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MultiPageLoaderPrivate *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->fail(); break;
        case 1: _t->loadDone(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject wkhtmltopdf::MultiPageLoaderPrivate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_wkhtmltopdf__MultiPageLoaderPrivate.data,
    qt_meta_data_wkhtmltopdf__MultiPageLoaderPrivate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *wkhtmltopdf::MultiPageLoaderPrivate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *wkhtmltopdf::MultiPageLoaderPrivate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_wkhtmltopdf__MultiPageLoaderPrivate.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int wkhtmltopdf::MultiPageLoaderPrivate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
